@@ -29,6 +29,7 @@
 #include "gdkmacoskeymap-private.h"
 #include "gdkmacossurface-private.h"
 #include "gdkmacosseat-private.h"
+#include "gdkmacosutils-private.h"
 
 #include "gdk/gdkeventsprivate.h"
 
@@ -592,6 +593,7 @@ fill_scroll_event (GdkMacosDisplay *self,
                    int              y)
 {
   GdkScrollDirection direction = 0;
+  GdkScrollRelativeDirection relative_direction;
   GdkModifierType state;
   GdkDevice *pointer;
   GdkEvent *ret = NULL;
@@ -606,6 +608,9 @@ fill_scroll_event (GdkMacosDisplay *self,
 
   phase = [nsevent phase];
   momentumPhase = [nsevent momentumPhase];
+  relative_direction = [nsevent isDirectionInvertedFromDevice] ?
+    GDK_SCROLL_RELATIVE_DIRECTION_INVERTED :
+    GDK_SCROLL_RELATIVE_DIRECTION_IDENTICAL;
 
   /* Ignore kinetic scroll events from the display server as we already
    * handle those internally.
@@ -628,7 +633,8 @@ fill_scroll_event (GdkMacosDisplay *self,
                                  get_time_from_ns_event (nsevent),
                                  state,
                                  0.0, 0.0, TRUE,
-                                 GDK_SCROLL_UNIT_SURFACE);
+                                 GDK_SCROLL_UNIT_SURFACE,
+                                 relative_direction);
 
   dx = [nsevent deltaX];
   dy = [nsevent deltaY];
@@ -650,7 +656,8 @@ fill_scroll_event (GdkMacosDisplay *self,
                                     -sx,
                                     -sy,
                                     FALSE,
-                                    GDK_SCROLL_UNIT_SURFACE);
+                                    GDK_SCROLL_UNIT_SURFACE,
+                                    relative_direction);
 
       /* Fall through for scroll emulation */
     }
@@ -683,7 +690,8 @@ fill_scroll_event (GdkMacosDisplay *self,
                                            NULL,
                                            get_time_from_ns_event (nsevent),
                                            state,
-                                           direction);
+                                           direction,
+                                           relative_direction);
     }
 
   if (phase == NSEventPhaseEnded || phase == NSEventPhaseCancelled)
@@ -699,7 +707,8 @@ fill_scroll_event (GdkMacosDisplay *self,
                                   get_time_from_ns_event (nsevent),
                                   state,
                                   0.0, 0.0, TRUE,
-                                  GDK_SCROLL_UNIT_SURFACE);
+                                  GDK_SCROLL_UNIT_SURFACE,
+                                  relative_direction);
     }
 
   return g_steal_pointer (&ret);
