@@ -20,14 +20,6 @@
 
 /* {{{ */
 
-#ifdef WINVER
-#undef WINVER
-#endif
-#ifdef _WIN32_WINNT
-#undef _WIN32_WINNT
-#endif
-#define WINVER 0x0603
-#define _WIN32_WINNT 0x0603
 #include "config.h"
 
 #include <gdk/gdk.h>
@@ -158,7 +150,8 @@ DManipEventHandler_OnContentUpdated (IDirectManipulationViewportEventHandler *se
 
   hr = IDirectManipulationContent_GetContentTransform (content, transform,
                                                        G_N_ELEMENTS (transform));
-  HR_CHECK_RETURN_VAL (hr, E_FAIL);
+  if G_UNLIKELY (FAILED (hr))
+    return E_FAIL;
 
   switch (self->gesture)
     {
@@ -186,7 +179,8 @@ DManipEventHandler_OnContentUpdated (IDirectManipulationViewportEventHandler *se
                                       (self->pan_x - pan_x) / scale,
                                       (self->pan_y - pan_y) / scale,
                                       FALSE,
-                                      GDK_SCROLL_UNIT_SURFACE);
+                                      GDK_SCROLL_UNIT_SURFACE,
+                                      GDK_SCROLL_RELATIVE_DIRECTION_UNKNOWN);
         _gdk_win32_append_event (event);
 
         self->pan_x = pan_x;
@@ -254,7 +248,8 @@ DManipEventHandler_OnViewportStatusChanged (IDirectManipulationViewportEventHand
             event = gdk_scroll_event_new (self->surface, self->device,
                                           NULL, time, state,
                                           0.0, 0.0, TRUE,
-                                          GDK_SCROLL_UNIT_SURFACE);
+                                          GDK_SCROLL_UNIT_SURFACE,
+                                          GDK_SCROLL_RELATIVE_DIRECTION_UNKNOWN);
             _gdk_win32_append_event (event);
           }
         break;
@@ -353,11 +348,13 @@ reset_viewport (IDirectManipulationViewport *viewport)
   HRESULT hr;
 
   hr = IDirectManipulationViewport_GetPrimaryContent (viewport, iid, (void**)&content);
-  HR_CHECK_GOTO (hr, failed);
+  if (G_UNLIKELY (FAILED (hr)))
+    goto failed;
 
   hr = IDirectManipulationContent_SyncContentTransform (content, identity,
                                                         G_N_ELEMENTS (identity));
-  HR_CHECK_GOTO (hr, failed);
+  if (G_UNLIKELY (FAILED (hr)))
+    goto failed;
 
 failed:
   gdk_win32_com_clear (&content);
@@ -413,7 +410,8 @@ create_viewport (GdkSurface *surface,
   hr = IDirectManipulationManager_CreateViewport (dmanipulation_manager, NULL, hwnd,
                                                   &IID_IDirectManipulationViewport,
                                                   (void**) pViewport);
-  HR_CHECK_GOTO (hr, failed);
+  if (G_UNLIKELY (FAILED (hr)))
+    goto failed;
 
   switch (gesture)
     {
@@ -435,16 +433,21 @@ create_viewport (GdkSurface *surface,
     dmanip_event_handler_new (surface, gesture);
 
   hr = IDirectManipulationViewport_AddEventHandler (*pViewport, hwnd, handler, &cookie);
-  HR_CHECK_GOTO (hr, failed);
+  if (G_UNLIKELY (FAILED (hr)))
+    goto failed;
 
   hr = IDirectManipulationViewport_ActivateConfiguration (*pViewport, configuration);
-  HR_CHECK_GOTO (hr, failed);
+  if (G_UNLIKELY (FAILED (hr)))
+    goto failed;
 
   hr = IDirectManipulationViewport_SetViewportOptions (*pViewport,
          DIRECTMANIPULATION_VIEWPORT_OPTIONS_DISABLEPIXELSNAPPING);
+  if (G_UNLIKELY (FAILED (hr)))
+    goto failed;
 
   hr = IDirectManipulationViewport_Enable (*pViewport);
-  HR_CHECK_GOTO (hr, failed);
+  if (G_UNLIKELY (FAILED (hr)))
+    goto failed;
 
   // drop our initial reference
   IUnknown_Release (handler);
@@ -500,10 +503,15 @@ void gdk_dmanipulation_initialize (GdkWin32Display *display)
 
       if (FAILED (hr))
         {
-          if (hr == REGDB_E_CLASSNOTREG || hr == E_NOINTERFACE);
-            /* Not an error,
-             * DirectManipulation is not available */
-          else HR_LOG (hr);
+          if (hr == REGDB_E_CLASSNOTREG || hr == E_NOINTERFACE)
+            {
+              /* Not an error,
+               * DirectManipulation is not available */
+            }
+          else
+            {
+              hr_warn (hr);
+            }
         }
     }
 }
@@ -527,7 +535,6 @@ void gdk_dmanipulation_initialize_surface (GdkSurface *surface)
 
   hr = IDirectManipulationManager_Activate (dmanipulation_manager,
                                             GDK_SURFACE_HWND (surface));
-  HR_CHECK_RETURN (hr);
 }
 
 void gdk_dmanipulation_finalize_surface (GdkSurface *surface)
@@ -564,15 +571,11 @@ void gdk_dmanipulation_maybe_add_contact (GdkSurface *surface,
   if (type == PT_TOUCHPAD)
     {
       GdkWin32Surface *surface_win32 = GDK_WIN32_SURFACE (surface);
-      HRESULT hr;
 
-      hr = IDirectManipulationViewport_SetContact (surface_win32->dmanipulation_viewport_pan,
-                                                   pointer_id);
-      HR_CHECK_RETURN (hr);
-
-      hr = IDirectManipulationViewport_SetContact (surface_win32->dmanipulation_viewport_zoom,
-                                                   pointer_id);
-      HR_CHECK_RETURN (hr);
+      IDirectManipulationViewport_SetContact (surface_win32->dmanipulation_viewport_pan,
+                                              pointer_id);
+      IDirectManipulationViewport_SetContact (surface_win32->dmanipulation_viewport_zoom,
+                                              pointer_id);
     }
 }
 
