@@ -2,7 +2,7 @@
 
 #include "gskgpushaderopprivate.h"
 
-#include "gskrendernode.h"
+#include "gskrendernodeprivate.h"
 
 #include <graphene.h>
 
@@ -10,12 +10,15 @@ G_BEGIN_DECLS
 
 void                    gsk_gpu_conic_gradient_op                       (GskGpuFrame                    *frame,
                                                                          GskGpuShaderClip                clip,
-                                                                         GskGpuColorStates               color_states,
+                                                                         GdkColorState                  *ccs,
+                                                                         float                           opacity,
+                                                                         const graphene_point_t         *offset,
+                                                                         GdkColorState                  *ics,
+                                                                         GskHueInterpolation             hue_interp,
                                                                          const graphene_rect_t          *rect,
                                                                          const graphene_point_t         *center,
                                                                          float                           angle,
-                                                                         const graphene_point_t         *offset,
-                                                                         const GskColorStop             *stops,
+                                                                         const GskGradientStop          *stops,
                                                                          gsize                           n_stops);
 
 
