@@ -4,6 +4,7 @@
 
 #include "gskglimageprivate.h"
 #include "gskgpuprintprivate.h"
+#include "gskgpuutilsprivate.h"
 #ifdef GDK_RENDERING_VULKAN
 #include "gskvulkanimageprivate.h"
 #endif
@@ -54,7 +55,7 @@ gsk_gpu_mipmap_op_vk_command (GskGpuOp              *op,
   vk_image = gsk_vulkan_image_get_vk_image (image);
   width = gsk_gpu_image_get_width (self->image);
   height = gsk_gpu_image_get_height (self->image);
-  n_levels = gsk_vulkan_mipmap_levels (width, height);
+  n_levels = gsk_gpu_mipmap_levels (width, height);
 
   /* optimize me: only transition mipmap layers 1..n, but not 0 */
   gsk_vulkan_image_transition (image,
@@ -156,8 +157,7 @@ gsk_gpu_mipmap_op_gl_command (GskGpuOp          *op,
 {
   GskGpuMipmapOp *self = (GskGpuMipmapOp *) op;
 
-  glActiveTexture (GL_TEXTURE0);
-  gsk_gl_image_bind_texture (GSK_GL_IMAGE (self->image));
+  gsk_gl_image_bind_textures (GSK_GL_IMAGE (self->image), GL_TEXTURE0);
   /* need to reset the images again */
   state->current_images[0] = NULL;
 
