@@ -15,8 +15,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_TREE_VIEW_H__
-#define __GTK_TREE_VIEW_H__
+#pragma once
 
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
@@ -37,6 +36,8 @@ G_BEGIN_DECLS
  * @GTK_TREE_VIEW_DROP_INTO_OR_AFTER: dropped row becomes a child or is inserted after
  *
  * An enum for determining where a dropped row goes.
+ *
+ * Deprecated: 4.20: There is no replacement.
  */
 typedef enum
 {
@@ -79,6 +80,8 @@ typedef struct _GtkTreeSelection      GtkTreeSelection;
  * spot to the user.
  *
  * Returns: %TRUE, if @column can be dropped in this spot
+ *
+ * Deprecated: 4.20: There is no replacement.
  */
 typedef gboolean (* GtkTreeViewColumnDropFunc) (GtkTreeView             *tree_view,
 						GtkTreeViewColumn       *column,
@@ -93,6 +96,8 @@ typedef gboolean (* GtkTreeViewColumnDropFunc) (GtkTreeView             *tree_vi
  * @user_data: user data
  *
  * Function used for gtk_tree_view_map_expanded_rows().
+ *
+ * Deprecated: 4.20: There is no replacement.
  */
 typedef void     (* GtkTreeViewMappingFunc)    (GtkTreeView             *tree_view,
 						GtkTreePath             *path,
@@ -113,6 +118,8 @@ typedef void     (* GtkTreeViewMappingFunc)    (GtkTreeView             *tree_vi
  * has some similarity to strcmp() returning 0 for equal strings.
  *
  * Returns: %FALSE if the row matches, %TRUE otherwise.
+ *
+ * Deprecated: 4.20: There is no replacement.
  */
 typedef gboolean (*GtkTreeViewSearchEqualFunc) (GtkTreeModel            *model,
 						int                      column,
@@ -132,6 +139,8 @@ typedef gboolean (*GtkTreeViewSearchEqualFunc) (GtkTreeModel            *model,
  * returns.
  *
  * Returns: %TRUE if the row is a separator
+ *
+ * Deprecated: 4.20: There is no replacement.
  */
 typedef gboolean (*GtkTreeViewRowSeparatorFunc) (GtkTreeModel      *model,
 						 GtkTreeIter       *iter,
@@ -545,4 +554,3 @@ G_DEFINE_AUTOPTR_CLEANUP_FUNC(GtkTreeView, g_object_unref)
 G_END_DECLS
 
 
-#endif /* __GTK_TREE_VIEW_H__ */
