@@ -392,6 +392,8 @@ gtk_adjustment_new (double value,
 		    double page_increment,
 		    double page_size)
 {
+  g_return_val_if_fail (lower + page_size <= upper, NULL);
+
   return g_object_new (GTK_TYPE_ADJUSTMENT,
 		       "lower", lower,
 		       "upper", upper,
@@ -558,6 +560,7 @@ gtk_adjustment_set_value (GtkAdjustment *adjustment,
 			  double         value)
 {
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (value));
 
   gtk_adjustment_set_value_internal (adjustment, value, FALSE);
 }
@@ -567,6 +570,7 @@ gtk_adjustment_animate_to_value (GtkAdjustment *adjustment,
 			         double         value)
 {
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (value));
 
   gtk_adjustment_set_value_internal (adjustment, value, TRUE);
 }
@@ -615,6 +619,7 @@ gtk_adjustment_set_lower (GtkAdjustment *adjustment,
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (lower));
 
   if (lower != priv->lower)
     {
@@ -662,6 +667,7 @@ gtk_adjustment_set_upper (GtkAdjustment *adjustment,
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (upper));
 
   if (upper != priv->upper)
     {
@@ -706,6 +712,7 @@ gtk_adjustment_set_step_increment (GtkAdjustment *adjustment,
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (step_increment));
 
   if (step_increment != priv->step_increment)
     {
@@ -750,6 +757,7 @@ gtk_adjustment_set_page_increment (GtkAdjustment *adjustment,
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (page_increment));
 
   if (page_increment != priv->page_increment)
     {
@@ -794,6 +802,7 @@ gtk_adjustment_set_page_size (GtkAdjustment *adjustment,
   GtkAdjustmentPrivate *priv = gtk_adjustment_get_instance_private (adjustment);
 
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (page_size));
 
   if (page_size != priv->page_size)
     {
@@ -833,6 +842,7 @@ gtk_adjustment_configure (GtkAdjustment *adjustment,
   gboolean value_changed = FALSE;
 
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (lower + page_size <= upper);
 
   g_object_freeze_notify (G_OBJECT (adjustment));
 
@@ -885,6 +895,7 @@ gtk_adjustment_clamp_page (GtkAdjustment *adjustment,
   gboolean need_emission;
 
   g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
+  g_return_if_fail (isfinite (lower) && isfinite (upper));
 
   lower = CLAMP (lower, priv->lower, priv->upper);
   upper = CLAMP (upper, priv->lower, priv->upper);
