@@ -280,8 +280,13 @@ gtk_column_view_scrollable_get_border (GtkScrollable *scrollable,
                                        GtkBorder     *border)
 {
   GtkColumnView *self = GTK_COLUMN_VIEW (scrollable);
+  int min, nat;
 
-  border->top = gtk_widget_get_height (self->header);
+  gtk_widget_measure (self->header, GTK_ORIENTATION_VERTICAL, -1, &min, &nat, NULL, NULL);
+  if (gtk_scrollable_get_vscroll_policy (GTK_SCROLLABLE (self->listview)) == GTK_SCROLL_MINIMUM)
+    border->top = min;
+  else
+    border->top = nat;
 
   return TRUE;
 }
@@ -943,7 +948,7 @@ gtk_column_view_class_init (GtkColumnViewClass *klass)
 
   /**
    * GtkColumnView::activate:
-   * @self: The `GtkColumnView`
+   * @self: The columnview
    * @position: position of item to activate
    *
    * Emitted when a row has been activated by the user, usually via activating
@@ -1543,7 +1548,7 @@ gtk_column_view_new (GtkSelectionModel *model)
 
 /**
  * gtk_column_view_get_model:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the model that's currently used to read the items displayed.
  *
@@ -1559,7 +1564,7 @@ gtk_column_view_get_model (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_model:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @model: (nullable) (transfer none): the model to use
  *
  * Sets the model to use.
@@ -1583,13 +1588,13 @@ gtk_column_view_set_model (GtkColumnView     *self,
 
 /**
  * gtk_column_view_get_columns:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the list of columns in this column view.
  *
  * This list is constant over the lifetime of @self and can be used to
  * monitor changes to the columns of @self by connecting to the
- * ::items-changed signal.
+ * [signal@Gio.ListModel::items-changed] signal.
  *
  * Returns: (transfer none): The list managing the columns
  */
@@ -1603,11 +1608,10 @@ gtk_column_view_get_columns (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_show_row_separators:
- * @self: a `GtkColumnView`
- * @show_row_separators: %TRUE to show row separators
+ * @self: a columnview
+ * @show_row_separators: whether to show row separators
  *
- * Sets whether the list should show separators
- * between rows.
+ * Sets whether the list should show separators between rows.
  */
 void
 gtk_column_view_set_show_row_separators (GtkColumnView *self,
@@ -1625,12 +1629,11 @@ gtk_column_view_set_show_row_separators (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_show_row_separators:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
- * Returns whether the list should show separators
- * between rows.
+ * Returns whether the list should show separators between rows.
  *
- * Returns: %TRUE if the list shows separators
+ * Returns: true if the list shows separators
  */
 gboolean
 gtk_column_view_get_show_row_separators (GtkColumnView *self)
@@ -1642,11 +1645,10 @@ gtk_column_view_get_show_row_separators (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_show_column_separators:
- * @self: a `GtkColumnView`
- * @show_column_separators: %TRUE to show column separators
+ * @self: a columnview
+ * @show_column_separators: whether to show column separators
  *
- * Sets whether the list should show separators
- * between columns.
+ * Sets whether the list should show separators between columns.
  */
 void
 gtk_column_view_set_show_column_separators (GtkColumnView *self,
@@ -1669,12 +1671,11 @@ gtk_column_view_set_show_column_separators (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_show_column_separators:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
- * Returns whether the list should show separators
- * between columns.
+ * Returns whether the list should show separators between columns.
  *
- * Returns: %TRUE if the list shows column separators
+ * Returns: true if the list shows column separators
  */
 gboolean
 gtk_column_view_get_show_column_separators (GtkColumnView *self)
@@ -1686,9 +1687,8 @@ gtk_column_view_get_show_column_separators (GtkColumnView *self)
 
 /**
  * gtk_column_view_append_column:
- * @self: a `GtkColumnView`
- * @column: a `GtkColumnViewColumn` that hasn't been added to a
- *   `GtkColumnView` yet
+ * @self: a columnview
+ * @column: a column that hasn't been added to a `GtkColumnView` yet
  *
  * Appends the @column to the end of the columns in @self.
  */
@@ -1706,8 +1706,8 @@ gtk_column_view_append_column (GtkColumnView       *self,
 
 /**
  * gtk_column_view_remove_column:
- * @self: a `GtkColumnView`
- * @column: a `GtkColumnViewColumn` that's part of @self
+ * @self: a columnview
+ * @column: a column that's part of @self
  *
  * Removes the @column from the list of columns of @self.
  */
@@ -1751,9 +1751,9 @@ gtk_column_view_remove_column (GtkColumnView       *self,
 
 /**
  * gtk_column_view_insert_column:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @position: the position to insert @column at
- * @column: the `GtkColumnViewColumn` to insert
+ * @column: the column to insert
  *
  * Inserts a column at the given position in the columns of @self.
  *
@@ -1892,7 +1892,7 @@ gtk_column_view_get_list_view (GtkColumnView *self)
 
 /**
  * gtk_column_view_get_sorter:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns a special sorter that reflects the users sorting
  * choices in the column view.
@@ -1926,8 +1926,8 @@ gtk_column_view_get_sorter (GtkColumnView *self)
 
 /**
  * gtk_column_view_sort_by_column:
- * @self: a `GtkColumnView`
- * @column: (nullable): the `GtkColumnViewColumn` to sort by
+ * @self: a columnview
+ * @column: (nullable): the column to sort by
  * @direction: the direction to sort in
  *
  * Sets the sorting of the view.
@@ -1941,7 +1941,7 @@ gtk_column_view_get_sorter (GtkColumnView *self)
  * and [method@Gtk.ColumnViewColumn.set_sorter] has been called
  * on @column to associate a sorter with the column.
  *
- * If @column is %NULL, the view will be unsorted.
+ * If @column is unset, the view will be unsorted.
  */
 void
 gtk_column_view_sort_by_column (GtkColumnView       *self,
@@ -1962,8 +1962,8 @@ gtk_column_view_sort_by_column (GtkColumnView       *self,
 
 /**
  * gtk_column_view_set_single_click_activate:
- * @self: a `GtkColumnView`
- * @single_click_activate: %TRUE to activate items on single click
+ * @self: a columnview
+ * @single_click_activate: whether to activate items on single click
  *
  * Sets whether rows should be activated on single click and
  * selected on hover.
@@ -1984,12 +1984,12 @@ gtk_column_view_set_single_click_activate (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_single_click_activate:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns whether rows will be activated on single click and
  * selected on hover.
  *
- * Returns: %TRUE if rows are activated on single click
+ * Returns: true if rows are activated on single click
  */
 gboolean
 gtk_column_view_get_single_click_activate (GtkColumnView *self)
@@ -2001,7 +2001,7 @@ gtk_column_view_get_single_click_activate (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_reorderable:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @reorderable: whether columns should be reorderable
  *
  * Sets whether columns should be reorderable by dragging.
@@ -2022,11 +2022,11 @@ gtk_column_view_set_reorderable (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_reorderable:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns whether columns are reorderable.
  *
- * Returns: %TRUE if columns are reorderable
+ * Returns: true if columns are reorderable
  */
 gboolean
 gtk_column_view_get_reorderable (GtkColumnView *self)
@@ -2038,8 +2038,8 @@ gtk_column_view_get_reorderable (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_enable_rubberband:
- * @self: a `GtkColumnView`
- * @enable_rubberband: %TRUE to enable rubberband selection
+ * @self: a columnview
+ * @enable_rubberband: whether to enable rubberband selection
  *
  * Sets whether selections can be changed by dragging with the mouse.
  */
@@ -2059,11 +2059,11 @@ gtk_column_view_set_enable_rubberband (GtkColumnView *self,
 
 /**
  * gtk_column_view_get_enable_rubberband:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Returns whether rows can be selected by dragging with the mouse.
  *
- * Returns: %TRUE if rubberband selection is enabled
+ * Returns: true if rubberband selection is enabled
  */
 gboolean
 gtk_column_view_get_enable_rubberband (GtkColumnView *self)
@@ -2075,16 +2075,19 @@ gtk_column_view_get_enable_rubberband (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_row_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @factory: (nullable): The row factory
  *
- * Sets the factory used for configuring rows. The factory must be for configuring
- * [class@Gtk.ColumnViewRow] objects.
+ * Sets the factory used for configuring rows.
  *
- * If this factory is not set - which is the default - then the defaults will be used.
+ * The factory must be for configuring [class@Gtk.ColumnViewRow] objects.
  *
- * This factory is not used to set the widgets displayed in the individual cells. For
- * that see [method@GtkColumnViewColumn.set_factory] and [class@GtkColumnViewCell].
+ * If this factory is not set - which is the default - then the defaults
+ * will be used.
+ *
+ * This factory is not used to set the widgets displayed in the individual
+ * cells. For that see [method@GtkColumnViewColumn.set_factory] and
+ * [class@GtkColumnViewCell].
  *
  * Since: 4.12
  */
@@ -2104,7 +2107,7 @@ gtk_column_view_set_row_factory (GtkColumnView      *self,
 
 /**
  * gtk_column_view_get_row_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the factory set via [method@Gtk.ColumnView.set_row_factory].
  *
@@ -2122,10 +2125,14 @@ gtk_column_view_get_row_factory (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_tab_behavior:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @tab_behavior: The desired tab behavior
  *
- * Sets the behavior of the <kbd>Tab</kbd> and <kbd>Shift</kbd>+<kbd>Tab</kbd> keys.
+ * Sets the <kbd>Tab</kbd> key behavior.
+ *
+ * This influences how the <kbd>Tab</kbd> and
+ * <kbd>Shift</kbd>+<kbd>Tab</kbd> keys move the
+ * focus in the columnview.
  *
  * Since: 4.12
  */
@@ -2145,7 +2152,7 @@ gtk_column_view_set_tab_behavior (GtkColumnView      *self,
 
 /**
  * gtk_column_view_get_tab_behavior:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the behavior set for the <kbd>Tab</kbd> key.
  *
@@ -2163,7 +2170,7 @@ gtk_column_view_get_tab_behavior (GtkColumnView *self)
 
 /**
  * gtk_column_view_get_header_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  *
  * Gets the factory that's currently used to populate section headers.
  *
@@ -2181,13 +2188,13 @@ gtk_column_view_get_header_factory (GtkColumnView *self)
 
 /**
  * gtk_column_view_set_header_factory:
- * @self: a `GtkColumnView`
+ * @self: a columnview
  * @factory: (nullable) (transfer none): the factory to use
  *
- * Sets the `GtkListItemFactory` to use for populating the
+ * Sets the factory to use for populating the
  * [class@Gtk.ListHeader] objects used in section headers.
  *
- * If this factory is set to %NULL, the list will not show
+ * If this factory is set to `NULL`, the list will not show
  * section headers.
  *
  * Since: 4.12
@@ -2209,11 +2216,11 @@ gtk_column_view_set_header_factory (GtkColumnView      *self,
 
 /**
  * gtk_column_view_scroll_to:
- * @self: The columnview to scroll in
+ * @self: The columnview
  * @pos: position of the item. Must be less than the number of
  *   items in the view.
  * @column: (nullable) (transfer none): The column to scroll to
- *   or %NULL to not scroll columns.
+ *   or `NULL` to not scroll columns
  * @flags: actions to perform
  * @scroll: (nullable) (transfer full): details of how to perform
  *   the scroll operation or %NULL to scroll into view
@@ -2221,7 +2228,7 @@ gtk_column_view_set_header_factory (GtkColumnView      *self,
  * Scroll to the row at the given position - or cell if a column is
  * given - and performs the actions specified in @flags.
  *
- * This function works no matter if the listview is shown or focused.
+ * This function works no matter if the columnview is shown or focused.
  * If it isn't, then the changes will take effect once that happens.
  *
  * Since: 4.12
@@ -2254,4 +2261,3 @@ gtk_column_view_scroll_to (GtkColumnView       *self,
   else
     g_clear_pointer (&scroll, gtk_scroll_info_unref);
 }
-
