@@ -17,8 +17,7 @@
  * Authors: Benjamin Otte <otte@gnome.org>
  */
 
-#ifndef __GTK_INSCRIPTION_PRIVATE_H__
-#define __GTK_INSCRIPTION_PRIVATE_H__
+#pragma once
 
 
 #include <gtk/gtkinscription.h>
@@ -28,6 +27,10 @@ G_BEGIN_DECLS
 
 PangoLayout *           gtk_inscription_get_layout              (GtkInscription *self);
 
+void
+gtk_inscription_get_layout_location (GtkInscription *self,
+                                     float          *x_out,
+                                     float          *y_out);
+
 G_END_DECLS
 
-#endif /* __GTK_INSCRIPTION_PRIVATE_H__ */
