@@ -22,13 +22,13 @@
 #include "gtkmultiselection.h"
 
 #include "gtkbitset.h"
+#include "gtksectionmodelprivate.h"
 #include "gtkselectionmodel.h"
 
 /**
  * GtkMultiSelection:
  *
- * `GtkMultiSelection` is a `GtkSelectionModel` that allows selecting multiple
- * elements.
+ * A selection model that allows selecting multiple elements.
  */
 
 struct _GtkMultiSelection
@@ -94,6 +94,23 @@ gtk_multi_selection_list_model_init (GListModelInterface *iface)
   iface->get_item = gtk_multi_selection_get_item;
 }
 
+static void
+gtk_multi_selection_get_section (GtkSectionModel *model,
+                                 guint            position,
+                                 guint           *out_start,
+                                 guint           *out_end)
+{
+  GtkMultiSelection *self = GTK_MULTI_SELECTION (model);
+
+  gtk_list_model_get_section (self->model, position, out_start, out_end);
+}
+
+static void
+gtk_multi_selection_section_model_init (GtkSectionModelInterface *iface)
+{
+  iface->get_section = gtk_multi_selection_get_section;
+}
+
 static gboolean
 gtk_multi_selection_is_selected (GtkSelectionModel *model,
                                  guint              position)
@@ -205,6 +222,8 @@ gtk_multi_selection_selection_model_init (GtkSelectionModelInterface *iface)
 G_DEFINE_TYPE_EXTENDED (GtkMultiSelection, gtk_multi_selection, G_TYPE_OBJECT, 0,
                         G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL,
                                                gtk_multi_selection_list_model_init)
+                        G_IMPLEMENT_INTERFACE (GTK_TYPE_SECTION_MODEL,
+                                               gtk_multi_selection_section_model_init)
                         G_IMPLEMENT_INTERFACE (GTK_TYPE_SELECTION_MODEL,
                                                gtk_multi_selection_selection_model_init))
 
@@ -271,6 +290,17 @@ gtk_multi_selection_items_changed_cb (GListModel        *model,
     g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_N_ITEMS]);
 }
 
+static void
+gtk_multi_selection_sections_changed_cb (GtkSectionModel *model,
+                                         unsigned int     position,
+                                         unsigned int     n_items,
+                                         gpointer         user_data)
+{
+  GtkMultiSelection *self = GTK_MULTI_SELECTION (user_data);
+
+  gtk_section_model_sections_changed (GTK_SECTION_MODEL (self), position, n_items);
+}
+
 static void
 gtk_multi_selection_clear_model (GtkMultiSelection *self)
 {
@@ -280,6 +310,9 @@ gtk_multi_selection_clear_model (GtkMultiSelection *self)
   g_signal_handlers_disconnect_by_func (self->model,
                                         gtk_multi_selection_items_changed_cb,
                                         self);
+  g_signal_handlers_disconnect_by_func (self->model,
+                                        gtk_multi_selection_sections_changed_cb,
+                                        self);
   g_clear_object (&self->model);
 }
 
@@ -367,7 +400,7 @@ gtk_multi_selection_class_init (GtkMultiSelectionClass *klass)
                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkMultiSelection:model: (attributes org.gtk.Property.get=gtk_multi_selection_get_model org.gtk.Property.set=gtk_multi_selection_set_model)
+   * GtkMultiSelection:model:
    *
    * The list managed by this selection.
    */
@@ -424,7 +457,7 @@ gtk_multi_selection_new (GListModel *model)
 }
 
 /**
- * gtk_multi_selection_get_model: (attributes org.gtk.Method.get_property=model)
+ * gtk_multi_selection_get_model:
  * @self: a `GtkMultiSelection`
  *
  * Returns the underlying model of @self.
@@ -440,7 +473,7 @@ gtk_multi_selection_get_model (GtkMultiSelection *self)
 }
 
 /**
- * gtk_multi_selection_set_model: (attributes org.gtk.Method.set_property=model)
+ * gtk_multi_selection_set_model:
  * @self: a `GtkMultiSelection`
  * @model: (nullable): A `GListModel` to wrap
  *
@@ -470,6 +503,9 @@ gtk_multi_selection_set_model (GtkMultiSelection *self,
                         "items-changed",
                         G_CALLBACK (gtk_multi_selection_items_changed_cb),
                         self);
+      if (GTK_IS_SECTION_MODEL (self->model))
+        g_signal_connect (self->model, "sections-changed",
+                          G_CALLBACK (gtk_multi_selection_sections_changed_cb), self);
       gtk_multi_selection_items_changed_cb (self->model,
                                             0,
                                             n_items_before,
