@@ -34,7 +34,7 @@
 /**
  * GtkTreeExpander:
  *
- * `GtkTreeExpander` is a widget that provides an expander for a list.
+ * Provides an expander for a tree-like list.
  *
  * It is typically placed as a bottommost child into a `GtkListView`
  * to allow users to expand and collapse children in a list with a
@@ -45,6 +45,11 @@
  * "listitem.toggle-expand" actions are provided to allow adding custom
  * UI for managing expanded state.
  *
+ * It is important to mention that you want to set the
+ * [property@Gtk.ListItem:focusable] property to FALSE when using this
+ * widget, as you want the keyboard focus to be in the treexpander, and not
+ * inside the list to make use of the keybindings.
+ *
  * The `GtkTreeListModel` must be set to not be passthrough. Then it
  * will provide [class@Gtk.TreeListRow] items which can be set via
  * [method@Gtk.TreeExpander.set_list_row] on the expander.
@@ -52,7 +57,38 @@
  * [method@Gtk.TreeExpander.set_child] sets the widget that displays
  * the actual row contents.
  *
- * # CSS nodes
+ * `GtkTreeExpander` can be modified with properties such as
+ * [property@Gtk.TreeExpander:indent-for-icon],
+ * [property@Gtk.TreeExpander:indent-for-depth], and
+ * [property@Gtk.TreeExpander:hide-expander] to achieve a different appearance.
+ * This can even be done to influence individual rows, for example by binding
+ * the [property@Gtk.TreeExpander:hide-expander] property to the item count of
+ * the model of the treelistrow, to hide the expander for rows without children,
+ * even if the row is expandable.
+ *
+ * ## Shortcuts and Gestures
+ *
+ * `GtkTreeExpander` supports the following keyboard shortcuts:
+ *
+ * - <kbd>+</kbd> or <kbd>*</kbd> expands the expander.
+ * - <kbd>-</kbd> or <kbd>/</kbd> collapses the expander.
+ * - Left and right arrow keys, when combined with <kbd>Shift</kbd> or
+ *   <kbd>Ctrl</kbd>+<kbd>Shift</kbd>, will expand or collapse, depending on
+ *   the locale's text direction.
+ * - <kbd>Ctrl</kbd>+<kbd>␣</kbd> toggles the expander state.
+ *
+ * The row can also expand on drag gestures.
+ *
+ * ## Actions
+ *
+ * `GtkTreeExpander` defines a set of built-in actions:
+ *
+ * - `listitem.expand` expands the expander if it can be expanded.
+ * - `listitem.collapse` collapses the expander.
+ * - `listitem.toggle-expand` tries to expand the expander if it was collapsed
+ *   or collapses it if it was expanded.
+ *
+ * ## CSS nodes
  *
  * ```
  * treeexpander
@@ -68,11 +104,12 @@
  *
  * For every level of depth, another "indent" node is prepended.
  *
- * # Accessibility
+ * ## Accessibility
  *
- * `GtkTreeExpander` uses the %GTK_ACCESSIBLE_ROLE_GROUP role. The expander icon
- * is represented as a %GTK_ACCESSIBLE_ROLE_BUTTON, labelled by the expander's
- * child, and toggling it will change the %GTK_ACCESSIBLE_STATE_EXPANDED state.
+ * Until GTK 4.10, `GtkTreeExpander` used the [enum@Gtk.AccessibleRole.group] role.
+ *
+ * Since GTK 4.12, `GtkTreeExpander` uses the [enum@Gtk.AccessibleRole.button] role.
+ * Toggling it will change the `GTK_ACCESSIBLE_STATE_EXPANDED` state.
  */
 
 struct _GtkTreeExpander
@@ -85,6 +122,8 @@ struct _GtkTreeExpander
   GtkWidget *expander_icon;
   guint notify_handler;
 
+  gboolean hide_expander;
+  gboolean indent_for_depth;
   gboolean indent_for_icon;
 
   guint expand_timer;
@@ -94,9 +133,11 @@ enum
 {
   PROP_0,
   PROP_CHILD,
+  PROP_HIDE_EXPANDER,
+  PROP_INDENT_FOR_DEPTH,
+  PROP_INDENT_FOR_ICON,
   PROP_ITEM,
   PROP_LIST_ROW,
-  PROP_INDENT_FOR_ICON,
 
   N_PROPS
 };
@@ -105,23 +146,6 @@ G_DEFINE_TYPE (GtkTreeExpander, gtk_tree_expander, GTK_TYPE_WIDGET)
 
 static GParamSpec *properties[N_PROPS] = { NULL, };
 
-static void
-gtk_tree_expander_click_gesture_pressed (GtkGestureClick *gesture,
-                                         int              n_press,
-                                         double           x,
-                                         double           y,
-                                         gpointer         unused)
-{
-  GtkWidget *widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (gesture));
-
-  gtk_widget_activate_action (widget, "listitem.toggle-expand", NULL);
-
-  gtk_widget_set_state_flags (widget,
-                              GTK_STATE_FLAG_ACTIVE,
-                              FALSE);
-
-  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-}
 
 static void
 gtk_tree_expander_click_gesture_released (GtkGestureClick *gesture,
@@ -130,21 +154,13 @@ gtk_tree_expander_click_gesture_released (GtkGestureClick *gesture,
                                           double           y,
                                           gpointer         unused)
 {
-  gtk_widget_unset_state_flags (gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (gesture)),
-                                GTK_STATE_FLAG_ACTIVE);
+  GtkWidget *widget = gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (gesture));
 
-  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-}
-
-static void
-gtk_tree_expander_click_gesture_canceled (GtkGestureClick  *gesture,
-                                          GdkEventSequence *sequence,
-                                          gpointer          unused)
-{
-  gtk_widget_unset_state_flags (gtk_event_controller_get_widget (GTK_EVENT_CONTROLLER (gesture)),
-                                GTK_STATE_FLAG_ACTIVE);
-
-  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
+  if (gtk_widget_contains (widget, x, y))
+    {
+      gtk_widget_activate_action (widget, "listitem.toggle-expand", NULL);
+      gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
+    }
 }
 
 static void
@@ -169,8 +185,8 @@ gtk_tree_expander_update_for_list_row (GtkTreeExpander *self)
       GtkWidget *child;
       guint i, depth;
 
-      depth = gtk_tree_list_row_get_depth (self->list_row);
-      if (gtk_tree_list_row_is_expandable (self->list_row))
+      depth = self->indent_for_depth ? gtk_tree_list_row_get_depth (self->list_row) : 0;
+      if (gtk_tree_list_row_is_expandable (self->list_row) && !self->hide_expander)
         {
           if (self->expander_icon == NULL)
             {
@@ -179,7 +195,7 @@ gtk_tree_expander_update_for_list_row (GtkTreeExpander *self)
               self->expander_icon =
                 g_object_new (GTK_TYPE_BUILTIN_ICON,
                               "css-name", "expander",
-                              "accessible-role", GTK_ACCESSIBLE_ROLE_BUTTON,
+                              "accessible-role", GTK_ACCESSIBLE_ROLE_NONE,
                               NULL);
 
               gesture = gtk_gesture_click_new ();
@@ -189,21 +205,13 @@ gtk_tree_expander_update_for_list_row (GtkTreeExpander *self)
                                                  FALSE);
               gtk_gesture_single_set_button (GTK_GESTURE_SINGLE (gesture),
                                              GDK_BUTTON_PRIMARY);
-              g_signal_connect (gesture, "pressed",
-                                G_CALLBACK (gtk_tree_expander_click_gesture_pressed), NULL);
               g_signal_connect (gesture, "released",
                                 G_CALLBACK (gtk_tree_expander_click_gesture_released), NULL);
-              g_signal_connect (gesture, "cancel",
-                                G_CALLBACK (gtk_tree_expander_click_gesture_canceled), NULL);
               gtk_widget_add_controller (self->expander_icon, GTK_EVENT_CONTROLLER (gesture));
 
               gtk_widget_insert_before (self->expander_icon,
                                         GTK_WIDGET (self),
                                         self->child);
-
-              gtk_accessible_update_property (GTK_ACCESSIBLE (self->expander_icon),
-                                              GTK_ACCESSIBLE_PROPERTY_LABEL, _("Expand"),
-                                              -1);
             }
 
           if (gtk_tree_list_row_get_expanded (self->list_row))
@@ -394,6 +402,18 @@ gtk_tree_expander_get_property (GObject    *object,
       g_value_set_object (value, self->child);
       break;
 
+    case PROP_HIDE_EXPANDER:
+      g_value_set_boolean (value, gtk_tree_expander_get_hide_expander (self));
+      break;
+
+    case PROP_INDENT_FOR_DEPTH:
+      g_value_set_boolean (value, gtk_tree_expander_get_indent_for_depth (self));
+      break;
+
+    case PROP_INDENT_FOR_ICON:
+      g_value_set_boolean (value, gtk_tree_expander_get_indent_for_icon (self));
+      break;
+
     case PROP_ITEM:
       g_value_take_object (value, gtk_tree_expander_get_item (self));
       break;
@@ -402,10 +422,6 @@ gtk_tree_expander_get_property (GObject    *object,
       g_value_set_object (value, self->list_row);
       break;
 
-    case PROP_INDENT_FOR_ICON:
-      g_value_set_boolean (value, gtk_tree_expander_get_indent_for_icon (self));
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -426,14 +442,22 @@ gtk_tree_expander_set_property (GObject      *object,
       gtk_tree_expander_set_child (self, g_value_get_object (value));
       break;
 
-    case PROP_LIST_ROW:
-      gtk_tree_expander_set_list_row (self, g_value_get_object (value));
+    case PROP_HIDE_EXPANDER:
+      gtk_tree_expander_set_hide_expander (self, g_value_get_boolean (value));
+      break;
+
+    case PROP_INDENT_FOR_DEPTH:
+      gtk_tree_expander_set_indent_for_depth (self, g_value_get_boolean (value));
       break;
 
     case PROP_INDENT_FOR_ICON:
       gtk_tree_expander_set_indent_for_icon (self, g_value_get_boolean (value));
       break;
 
+    case PROP_LIST_ROW:
+      gtk_tree_expander_set_list_row (self, g_value_get_object (value));
+      break;
+
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -472,11 +496,17 @@ gtk_tree_expander_toggle_expand (GtkWidget  *widget,
                                  GVariant   *parameter)
 {
   GtkTreeExpander *self = GTK_TREE_EXPANDER (widget);
+  gboolean expand;
 
   if (self->list_row == NULL)
     return;
 
-  gtk_tree_list_row_set_expanded (self->list_row, !gtk_tree_list_row_get_expanded (self->list_row));
+  expand = !gtk_tree_list_row_get_expanded (self->list_row);
+
+  if (expand)
+    gtk_widget_activate_action (widget, "listitem.scroll-to", NULL);
+
+  gtk_tree_list_row_set_expanded (self->list_row, expand);
 }
 
 static gboolean
@@ -523,7 +553,7 @@ gtk_tree_expander_class_init (GtkTreeExpanderClass *klass)
   gobject_class->set_property = gtk_tree_expander_set_property;
 
   /**
-   * GtkTreeExpander:child: (attributes org.gtk.Property.get=gtk_tree_expander_get_child org.gtk.Property.set=gtk_tree_expander_set_child)
+   * GtkTreeExpander:child:
    *
    * The child widget with the actual contents.
    */
@@ -533,27 +563,36 @@ gtk_tree_expander_class_init (GtkTreeExpanderClass *klass)
                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkTreeExpander:item: (attributes org.gtk.Property.get=gtk_tree_expander_get_item)
+   * GtkTreeExpander:hide-expander:
    *
-   * The item held by this expander's row.
+   * Whether the expander icon should be hidden in a GtkTreeListRow.
+   * Note that this property simply hides the icon.  The actions and keybinding
+   * (i.e. collapse and expand) are not affected by this property.
+   *
+   * A common use for this property would be to bind to the number of children in a
+   * GtkTreeListRow's model in order to hide the expander when a row has no children.
+   *
+   * Since: 4.10
    */
-  properties[PROP_ITEM] =
-      g_param_spec_object ("item", NULL, NULL,
-                           G_TYPE_OBJECT,
-                           G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+  properties[PROP_HIDE_EXPANDER] =
+      g_param_spec_boolean ("hide-expander", NULL, NULL,
+                            FALSE,
+                            G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkTreeExpander:list-row: (attributes org.gtk.Property.get=gtk_tree_expander_get_list_row org.gtk.Property.set=gtk_tree_expander_set_list_row)
+   * GtkTreeExpander:indent-for-depth:
    *
-   * The list row to track for expander state.
+   * TreeExpander indents the child according to its depth.
+   *
+   * Since: 4.10
    */
-  properties[PROP_LIST_ROW] =
-    g_param_spec_object ("list-row", NULL, NULL,
-                         GTK_TYPE_TREE_LIST_ROW,
-                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+  properties[PROP_INDENT_FOR_DEPTH] =
+      g_param_spec_boolean ("indent-for-depth", NULL, NULL,
+                            TRUE,
+                            G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkTreeExpander:indent-for-icon: (attributes org.gtk.Property.get=gtk_tree_expander_get_indent_for_icon org.gtk.Property.set=gtk_tree_expander_set_indent_for_icon)
+   * GtkTreeExpander:indent-for-icon:
    *
    * TreeExpander indents the child by the width of an expander-icon if it is not expandable.
    *
@@ -564,6 +603,26 @@ gtk_tree_expander_class_init (GtkTreeExpanderClass *klass)
                             TRUE,
                             G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * GtkTreeExpander:item:
+   *
+   * The item held by this expander's row.
+   */
+  properties[PROP_ITEM] =
+      g_param_spec_object ("item", NULL, NULL,
+                           G_TYPE_OBJECT,
+                           G_PARAM_READABLE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  /**
+   * GtkTreeExpander:list-row:
+   *
+   * The list row to track for expander state.
+   */
+  properties[PROP_LIST_ROW] =
+    g_param_spec_object ("list-row", NULL, NULL,
+                         GTK_TYPE_TREE_LIST_ROW,
+                         G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (gobject_class, N_PROPS, properties);
 
   /**
@@ -644,7 +703,7 @@ gtk_tree_expander_class_init (GtkTreeExpanderClass *klass)
 
   gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BOX_LAYOUT);
   gtk_widget_class_set_css_name (widget_class, I_("treeexpander"));
-  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_GROUP);
+  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_BUTTON);
 }
 
 static gboolean
@@ -697,6 +756,7 @@ gtk_tree_expander_init (GtkTreeExpander *self)
 
   gtk_widget_set_focusable (GTK_WIDGET (self), TRUE);
   self->indent_for_icon = TRUE;
+  self->indent_for_depth = TRUE;
 
   controller = gtk_drop_controller_motion_new ();
   g_signal_connect (controller, "enter", G_CALLBACK (gtk_tree_expander_drag_enter), self);
@@ -719,7 +779,7 @@ gtk_tree_expander_new (void)
 }
 
 /**
- * gtk_tree_expander_get_child: (attributes org.gtk.Method.get_property=child)
+ * gtk_tree_expander_get_child:
  * @self: a `GtkTreeExpander`
  *
  * Gets the child widget displayed by @self.
@@ -735,7 +795,7 @@ gtk_tree_expander_get_child (GtkTreeExpander *self)
 }
 
 /**
- * gtk_tree_expander_set_child: (attributes org.gtk.Method.set_property=child)
+ * gtk_tree_expander_set_child:
  * @self: a `GtkTreeExpander`
  * @child: (nullable): a `GtkWidget`
  *
@@ -746,7 +806,7 @@ gtk_tree_expander_set_child (GtkTreeExpander *self,
                              GtkWidget       *child)
 {
   g_return_if_fail (GTK_IS_TREE_EXPANDER (self));
-  g_return_if_fail (child == NULL || GTK_IS_WIDGET (child));
+  g_return_if_fail (child == NULL || self->child == child || gtk_widget_get_parent (child) == NULL);
 
   if (self->child == child)
     return;
@@ -771,7 +831,7 @@ gtk_tree_expander_set_child (GtkTreeExpander *self,
 }
 
 /**
- * gtk_tree_expander_get_item: (attributes org.gtk.Method.get_property=item)
+ * gtk_tree_expander_get_item:
  * @self: a `GtkTreeExpander`
  *
  * Forwards the item set on the `GtkTreeListRow` that @self is managing.
@@ -796,7 +856,7 @@ gtk_tree_expander_get_item (GtkTreeExpander *self)
 }
 
 /**
- * gtk_tree_expander_get_list_row: (attributes org.gtk.Method.get_property=list-row)
+ * gtk_tree_expander_get_list_row:
  * @self: a `GtkTreeExpander`
  *
  * Gets the list row managed by @self.
@@ -812,7 +872,7 @@ gtk_tree_expander_get_list_row (GtkTreeExpander *self)
 }
 
 /**
- * gtk_tree_expander_set_list_row: (attributes org.gtk.Method.set_property=list-row)
+ * gtk_tree_expander_set_list_row:
  * @self: a `GtkTreeExpander` widget
  * @list_row: (nullable): a `GtkTreeListRow`
  *
@@ -850,7 +910,50 @@ gtk_tree_expander_set_list_row (GtkTreeExpander *self,
 }
 
 /**
- * gtk_tree_expander_get_indent_for_icon: (attributes org.gtk.Method.get_property=indent-for-icon)
+ * gtk_tree_expander_get_indent_for_depth:
+ * @self: a `GtkTreeExpander`
+ *
+ * TreeExpander indents each level of depth with an additional indent.
+ *
+ * Returns: TRUE if the child should be indented . Otherwise FALSE.
+ *
+ * Since: 4.10
+ */
+gboolean
+gtk_tree_expander_get_indent_for_depth (GtkTreeExpander *self)
+{
+  g_return_val_if_fail (GTK_IS_TREE_EXPANDER (self), FALSE);
+
+  return self->indent_for_depth;
+}
+
+/**
+ * gtk_tree_expander_set_indent_for_depth:
+ * @self: a `GtkTreeExpander` widget
+ * @indent_for_depth: TRUE if the child should be indented. Otherwise FALSE.
+ *
+ * Sets if the TreeExpander should indent the child according to its depth.
+ *
+ * Since: 4.10
+ */
+void
+gtk_tree_expander_set_indent_for_depth (GtkTreeExpander *self,
+                                        gboolean         indent_for_depth)
+{
+  g_return_if_fail (GTK_IS_TREE_EXPANDER (self));
+
+  if (indent_for_depth == self->indent_for_depth)
+    return;
+
+  self->indent_for_depth = indent_for_depth;
+
+  gtk_tree_expander_update_for_list_row (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_INDENT_FOR_DEPTH]);
+}
+
+/**
+ * gtk_tree_expander_get_indent_for_icon:
  * @self: a `GtkTreeExpander`
  *
  * TreeExpander indents the child by the width of an expander-icon if it is not expandable.
@@ -868,7 +971,7 @@ gtk_tree_expander_get_indent_for_icon (GtkTreeExpander *self)
 }
 
 /**
- * gtk_tree_expander_set_indent_for_icon: (attributes org.gtk.Method.set_property=indent-for-icon)
+ * gtk_tree_expander_set_indent_for_icon:
  * @self: a `GtkTreeExpander` widget
  * @indent_for_icon: TRUE if the child should be indented without expander. Otherwise FALSE.
  *
@@ -878,7 +981,7 @@ gtk_tree_expander_get_indent_for_icon (GtkTreeExpander *self)
  */
 void
 gtk_tree_expander_set_indent_for_icon (GtkTreeExpander *self,
-                                       gboolean indent_for_icon)
+                                       gboolean         indent_for_icon)
 {
   g_return_if_fail (GTK_IS_TREE_EXPANDER (self));
 
@@ -891,3 +994,46 @@ gtk_tree_expander_set_indent_for_icon (GtkTreeExpander *self,
 
   g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_INDENT_FOR_ICON]);
 }
+
+/**
+ * gtk_tree_expander_get_hide_expander:
+ * @self: a `GtkTreeExpander`
+ *
+ * Gets whether the TreeExpander should be hidden in a GtkTreeListRow.
+ *
+ * Returns: TRUE if the expander icon should be hidden. Otherwise FALSE.
+ *
+ * Since: 4.10
+ */
+gboolean
+gtk_tree_expander_get_hide_expander (GtkTreeExpander *self)
+{
+  g_return_val_if_fail (GTK_IS_TREE_EXPANDER (self), FALSE);
+
+  return self->hide_expander;
+}
+
+/**
+ * gtk_tree_expander_set_hide_expander:
+ * @self: a `GtkTreeExpander` widget
+ * @hide_expander: TRUE if the expander should be hidden. Otherwise FALSE.
+ *
+ * Sets whether the expander icon should be visible in a GtkTreeListRow.
+ *
+ * Since: 4.10
+ */
+void
+gtk_tree_expander_set_hide_expander (GtkTreeExpander *self,
+                                     gboolean         hide_expander)
+{
+  g_return_if_fail (GTK_IS_TREE_EXPANDER (self));
+
+  if (hide_expander == self->hide_expander)
+    return;
+
+  self->hide_expander = hide_expander;
+
+  gtk_tree_expander_update_for_list_row (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_HIDE_EXPANDER]);
+}
