@@ -24,6 +24,8 @@
 #include <gtk/gtk.h>
 
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 static gboolean
 monospace_filter (const PangoFontFamily *family,
                   const PangoFontFace   *face,
@@ -161,7 +163,7 @@ main (int argc, char *argv[])
   g_signal_connect (toggle, "notify::active", G_CALLBACK (level_changed), font_button);
   gtk_box_append (GTK_BOX (box), toggle);
 
-  gtk_widget_show (window);
+  gtk_window_present (GTK_WINDOW (window));
 
   g_signal_connect (font_button, "notify::font",
                     G_CALLBACK (notify_font_cb), NULL);
