@@ -28,7 +28,7 @@ main (int argc, char *argv[])
   view = gtk_places_view_new ();
 
   gtk_window_set_child (GTK_WINDOW (win), view);
-  gtk_widget_show (win);
+  gtk_window_present (GTK_WINDOW (win));
 
   g_signal_connect (win, "destroy", G_CALLBACK (quit_cb), &done);
 
