@@ -19,6 +19,8 @@
 #include "config.h"
 #include <gtk/gtk.h>
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 /*
  * README README README README README README README README README README
  * README README README README README README README README README README
@@ -428,7 +430,6 @@ update_columns (GtkTreeView *view, ViewColumnModel *view_model)
       int *new_order;
       GtkTreePath *path;
 
-      new_order = g_new (int, length);
       a = old_columns; b = view_model->columns;
 
       while (a->data == b->data)
@@ -440,6 +441,8 @@ update_columns (GtkTreeView *view, ViewColumnModel *view_model)
 	  m++;
 	}
 
+      new_order = g_new (int, length);
+
       if (a->next->data == b->data)
 	{
 	  b = b->next;
@@ -764,7 +767,7 @@ main (int argc, char *argv[])
   swindow = gtk_scrolled_window_new ();
   gtk_window_set_child (GTK_WINDOW (window), swindow);
   gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (swindow), sample_tree_view_top);
-  gtk_widget_show (window);
+  gtk_window_present (GTK_WINDOW (window));
 
   window = gtk_window_new ();
   g_signal_connect (window, "destroy", G_CALLBACK (quit_cb), &done); 
@@ -773,7 +776,7 @@ main (int argc, char *argv[])
   swindow = gtk_scrolled_window_new ();
   gtk_window_set_child (GTK_WINDOW (window), swindow);
   gtk_scrolled_window_set_child (GTK_SCROLLED_WINDOW (swindow), sample_tree_view_bottom);
-  gtk_widget_show (window);
+  gtk_window_present (GTK_WINDOW (window));
 
   /* Set up the main window */
   window = gtk_window_new ();
@@ -910,7 +913,7 @@ main (int argc, char *argv[])
   g_signal_connect (button, "clicked", G_CALLBACK (add_clicked), left_tree_model);
   gtk_box_append (GTK_BOX (hbox), button);
 
-  gtk_widget_show (window);
+  gtk_window_present (GTK_WINDOW (window));
 
   while (!done)
     g_main_context_iteration (NULL, TRUE);
