@@ -111,7 +111,7 @@ load_ui_file (GFile *file, gboolean generate)
 
   reference_file = test_get_other_file (ui_file, ".nodes");
 
-  diff = diff_with_file (reference_file, output, -1, &error);
+  diff = diff_string_with_file (reference_file, output, -1, &error);
   g_assert_no_error (error);
 
   if (diff && diff[0])
