@@ -1,7 +1,11 @@
 #! /usr/bin/env python3
 
 import sys
-import gi
+
+try:
+    import gi
+except ImportError:
+    sys.exit(77) # skip this test, gi module is not available
 
 gi.require_version('Gtk', '4.0')
 
