@@ -40,12 +40,13 @@ usage (void)
              "Commands:\n"
              "  benchmark    Benchmark rendering of a node\n"
              "  compare      Compare nodes or images\n"
+             "  convert      Convert into nodes\n"
              "  extract      Extract data urls\n"
              "  info         Provide information about the node\n"
              "  show         Show the node\n"
              "  render       Take a screenshot of the node\n"
              "\n"));
-  exit (1);
+  exit (0);
 }
 
 static GLogWriterOutput
@@ -99,8 +100,6 @@ main (int argc, const char *argv[])
 
   gtk_init_check ();
 
-  gtk_test_register_all_types ();
-
   if (argc < 2)
     usage ();
 
@@ -122,6 +121,8 @@ main (int argc, const char *argv[])
     do_compare (&argc, &argv);
   else if (strcmp (argv[0], "extract") == 0)
     do_extract (&argc, &argv);
+  else if (strcmp (argv[0], "convert") == 0)
+    do_convert (&argc, &argv);
   else
     usage ();
 
